@@ -94,7 +94,7 @@ def _should_contain_zip_content(value):
     try:
         with closing(zipfile.ZipFile(fileobj)) as f:
             f.infolist()
-    except zipfile.BadZipfile:
+    except zipfile.BadZipFile:
         raise ValueError(ERROR_MSG)
 
 
